@@ -412,7 +412,7 @@ NI_ip_is_ipv4(const char *str)
     int len;
     int quads = 0;
     int quadspots[3];
-    int current_quad;
+    long current_quad;
     int cq_index;
     char *endptr;
 
@@ -494,7 +494,7 @@ NI_ip_is_ipv6(const char *str)
     int i;
     int len;
     int octs = 0;
-    int octspots[7];
+    int octspots[8];
     int oct_index;
     const char *double_colon;
     const char *next_oct;
@@ -2025,7 +2025,7 @@ NI_ip_normalize_prefix_ipv4(unsigned long ip, char *slash,
     unsigned long current;
     char *endptr = NULL;
     int res;
-    int clen   = 0;
+    long clen  = 0;
     int addcst = 0;
     char c;
 
@@ -2043,8 +2043,14 @@ NI_ip_normalize_prefix_ipv4(unsigned long ip, char *slash,
         if (STRTOL_FAILED(clen, slash, endptr)) {
             return 0;
         }
-
-        addcst = (*endptr == ',');
+        if (*endptr == ',') {
+            addcst = 1;
+        } else if (endptr != (slash + strlen(slash))) {
+	    NI_set_Error_Errno(172, "Invalid prefix length /%s", slash);
+            return 0;
+        } else {
+            addcst = 0;
+        }
 
         res = NI_ip_check_prefix_ipv4(current, clen);
         if (!res) {
@@ -2081,7 +2087,7 @@ NI_ip_normalize_prefix_ipv6(n128_t *ip, char *slash,
     n128_t current;
     char *endptr = NULL;
     int res;
-    int clen   = 0;
+    long clen  = 0;
     int addcst = 0;
     char c;
 
@@ -2099,8 +2105,14 @@ NI_ip_normalize_prefix_ipv6(n128_t *ip, char *slash,
         if (STRTOL_FAILED(clen, slash, endptr)) {
             return 0;
         }
-
-        addcst = (*endptr == ',');
+        if (*endptr == ',') {
+            addcst = 1;
+        } else if (endptr != (slash + strlen(slash))) {
+	    NI_set_Error_Errno(172, "Invalid prefix length /%s", slash);
+            return 0;
+        } else {
+            addcst = 0;
+        }
 
         res = NI_ip_check_prefix_ipv6(&current, clen);
         if (!res) {
@@ -2681,7 +2693,7 @@ NI_ip_splitprefix(const char *prefix, char *ipbuf, int *lenbuf)
     const char *c;
     const char *slash;
     char *endptr;
-    int num;
+    long num;
     int len;
 
     c = slash = strchr(prefix, '/');
